@@ -98,49 +98,22 @@ class Source(object):
         newsource.lines = [(indent+line) for line in self.lines]
         return newsource
 
-    def getstatement(self, lineno):
+    def getstatement(self, lineno, assertion=False):
         """ return Source statement which contains the
             given linenumber (counted from 0).
         """
-        start, end = self.getstatementrange(lineno)
+        start, end = self.getstatementrange(lineno, assertion)
         return self[start:end]
 
-    def getstatementrange(self, lineno):
+    def getstatementrange(self, lineno, assertion=False):
         """ return (start, end) tuple which spans the minimal
             statement region which containing the given lineno.
         """
-        # XXX there must be a better than these heuristic ways ...
-        # XXX there may even be better heuristics :-)
         if not (0 <= lineno < len(self)):
             raise IndexError("lineno out of range")
-
-        # 1. find the start of the statement
-        from codeop import compile_command
-        for start in range(lineno, -1, -1):
-            trylines = self.lines[start:lineno+1]
-            # quick hack to indent the source and get it as a string in one go
-            trylines.insert(0, 'def xxx():')
-            trysource = '\n '.join(trylines)
-            #              ^ space here
-            try:
-                compile_command(trysource)
-            except (SyntaxError, OverflowError, ValueError):
-                continue
-
-            # 2. find the end of the statement
-            for end in range(lineno+1, len(self)+1):
-                trysource = self[start:end]
-                if trysource.isparseable():
-                    return start, end
+        ast, start, end = getstatementrange_ast(lineno, self)
         return start, end
 
-    def getblockend(self, lineno):
-        # XXX
-        lines = [x + '\n' for x in self.lines[lineno:]]
-        blocklines = inspect.getblock(lines)
-        #print blocklines
-        return lineno + len(blocklines) - 1
-
     def deindent(self, offset=None):
         """ return a new source object deindented by offset.
             If offset is None then guess an indentation offset from
@@ -208,7 +181,7 @@ class Source(object):
             msglines = self.lines[:ex.lineno]
             if ex.offset:
                 msglines.append(" "*ex.offset + '^')
-            msglines.append("syntax error probably generated here: %s" % filename)
+            msglines.append("(code was compiled probably from here: %s)" % filename)
             newex = SyntaxError('\n'.join(msglines))
             newex.offset = ex.offset
             newex.lineno = ex.lineno
@@ -250,23 +223,29 @@ def compile_(source, filename=None, mode='exec', flags=
 
 
 def getfslineno(obj):
+    """ Return source location (path, lineno) for the given object.
+    If the source cannot be determined return ("", -1)
+    """
     try:
         code = py.code.Code(obj)
     except TypeError:
-        # fallback to
-        fn = (py.std.inspect.getsourcefile(obj) or
-              py.std.inspect.getfile(obj))
+        try:
+            fn = (py.std.inspect.getsourcefile(obj) or
+                  py.std.inspect.getfile(obj))
+        except TypeError:
+            return "", -1
+
         fspath = fn and py.path.local(fn) or None
+        lineno = -1
         if fspath:
             try:
                 _, lineno = findsource(obj)
             except IOError:
-                lineno = None
-        else:
-            lineno = None
+                pass
     else:
         fspath = code.path
         lineno = code.firstlineno
+    assert isinstance(lineno, int)
     return fspath, lineno
 
 #
@@ -279,7 +258,7 @@ def findsource(obj):
     except py.builtin._sysex:
         raise
     except:
-        return None, None
+        return None, -1
     source = Source()
     source.lines = [line.rstrip() for line in sourcelines]
     return source, lineno
@@ -312,14 +291,10 @@ def deindent(lines, offset=None):
         while True:
             yield ''
 
-    r = readline_generator(lines)
-    try:
-        readline = r.next
-    except AttributeError:
-        readline = r.__next__
+    it = readline_generator(lines)
 
     try:
-        for _, _, (sline, _), (eline, _), _ in tokenize.generate_tokens(readline):
+        for _, _, (sline, _), (eline, _), _ in tokenize.generate_tokens(lambda: next(it)):
             if sline > len(lines):
                 break # End of input reached
             if sline > len(newlines):
@@ -337,3 +312,124 @@ def deindent(lines, offset=None):
     # Add any lines we didn't see. E.g. if an exception was raised.
     newlines.extend(lines[len(newlines):])
     return newlines
+
+
+def get_statement_startend(lineno, nodelist):
+    from bisect import bisect_right
+    # lineno starts at 0
+    nextlineno = None
+    while 1:
+        lineno_list = [x.lineno-1 for x in nodelist] # ast indexes start at 1
+        #print lineno_list, [vars(x) for x in nodelist]
+        insert_index = bisect_right(lineno_list, lineno)
+        if insert_index >= len(nodelist):
+            insert_index -= 1
+        elif lineno < (nodelist[insert_index].lineno - 1) and insert_index > 0:
+            insert_index -= 1
+            assert lineno >= (nodelist[insert_index].lineno - 1)
+        nextnode = nodelist[insert_index]
+
+        try:
+            nextlineno = nodelist[insert_index+1].lineno - 1
+        except IndexError:
+            pass
+        lastnodelist = nodelist
+        nodelist = getnodelist(nextnode)
+        if not nodelist:
+            start, end = nextnode.lineno-1, nextlineno
+            start = min(lineno, start)
+            assert start <= lineno  and (end is None or lineno < end)
+            return start, end
+
+def getnodelist(node):
+    import _ast
+    l = []
+    #print "node", node, "fields", node._fields, "lineno", getattr(node, "lineno", 0) - 1
+    for subname in "test", "type", "body", "handlers", "orelse", "finalbody":
+        attr = getattr(node, subname, None)
+        if attr is not None:
+            if isinstance(attr, list):
+                l.extend(attr)
+            elif hasattr(attr, "lineno"):
+                l.append(attr)
+    return l
+
+def getstatementrange_ast(lineno, source, assertion=False, astnode=None):
+    if astnode is None:
+        content = str(source)
+        if sys.version_info < (2,7):
+            content += "\n"
+        try:
+            astnode = compile(content, "source", "exec", 1024)  # 1024 for AST
+        except ValueError:
+            start, end = getstatementrange_old(lineno, source, assertion)
+            return None, start, end
+    start, end = get_statement_startend(lineno, getnodelist(astnode))
+    # we need to correct the end:
+    # - ast-parsing strips comments
+    # - else statements do not have a separate lineno
+    # - there might be empty lines
+    # - we might have lesser indented code blocks at the end
+    if end is None:
+        end = len(source.lines)
+
+    if end > start + 1:
+        # make sure we don't span differently indented code blocks
+        # by using the BlockFinder helper used which inspect.getsource() uses itself
+        block_finder = inspect.BlockFinder()
+        # if we start with an indented line, put blockfinder to "started" mode
+        block_finder.started = source.lines[start][0].isspace()
+        it = ((x + "\n") for x in source.lines[start:end])
+        try:
+            for tok in tokenize.generate_tokens(lambda: next(it)):
+                block_finder.tokeneater(*tok)
+        except (inspect.EndOfBlock, IndentationError) as e:
+            end = block_finder.last + start
+        #except Exception:
+        #    pass
+
+    # the end might still point to a comment, correct it
+    while end:
+        line = source.lines[end - 1].lstrip()
+        if line.startswith("#"):
+            end -= 1
+        else:
+            break
+    return astnode, start, end
+
+
+def getstatementrange_old(lineno, source, assertion=False):
+    """ return (start, end) tuple which spans the minimal
+        statement region which containing the given lineno.
+        raise an IndexError if no such statementrange can be found.
+    """
+    # XXX this logic is only used on python2.4 and below
+    # 1. find the start of the statement
+    from codeop import compile_command
+    for start in range(lineno, -1, -1):
+        if assertion:
+            line = source.lines[start]
+            # the following lines are not fully tested, change with care
+            if 'super' in line and 'self' in line and '__init__' in line:
+                raise IndexError("likely a subclass")
+            if "assert" not in line and "raise" not in line:
+                continue
+        trylines = source.lines[start:lineno+1]
+        # quick hack to prepare parsing an indented line with
+        # compile_command() (which errors on "return" outside defs)
+        trylines.insert(0, 'def xxx():')
+        trysource = '\n '.join(trylines)
+        #              ^ space here
+        try:
+            compile_command(trysource)
+        except (SyntaxError, OverflowError, ValueError):
+            continue
+
+        # 2. find the end of the statement
+        for end in range(lineno+1, len(source)+1):
+            trysource = source[start:end]
+            if trysource.isparseable():
+                return start, end
+    raise SyntaxError("no valid source range around line %d " % (lineno,))
+
+
