@@ -7,11 +7,10 @@ from contextlib import contextmanager
 import sys, os, re, atexit, io
 import py
 from py._path import common
+from py._path.common import iswin32
 from stat import S_ISLNK, S_ISDIR, S_ISREG
 
-from os.path import abspath, normpath, isabs, exists, isdir, isfile, islink
-
-iswin32 = sys.platform == "win32" or (getattr(os, '_name', False) == 'nt')
+from os.path import abspath, normpath, isabs, exists, isdir, isfile, islink, dirname
 
 if sys.version_info > (3,0):
     def map_as_list(func, iter):
@@ -153,7 +152,7 @@ class LocalPath(FSBase):
         elif isinstance(path, py.builtin._basestring):
             if expanduser:
                 path = os.path.expanduser(path)
-            self.strpath = abspath(normpath(path))
+            self.strpath = abspath(path)
         else:
             raise ValueError("can only pass None, Path instances "
                              "or non-empty strings to LocalPath")
@@ -305,6 +304,16 @@ class LocalPath(FSBase):
                         raise ValueError("invalid part specification %r" % name)
         return res
 
+    def dirpath(self, *args, **kwargs):
+        """ return the directory path joined with any given path arguments.  """
+        if not kwargs:
+            path = object.__new__(self.__class__)
+            path.strpath = dirname(self.strpath)
+            if args:
+                path = path.join(*args)
+            return path
+        return super(LocalPath, self).dirpath(*args, **kwargs)
+
     def join(self, *args, **kwargs):
         """ return a new path by appending all 'args' as path
         components.  if abs=1 is used restart from root if any
@@ -571,35 +580,6 @@ class LocalPath(FSBase):
         """ return string representation of the Path. """
         return self.strpath
 
-    def pypkgpath(self, pkgname=None):
-        """ return the Python package path by looking for a
-            pkgname.  If pkgname is None look for the last
-            directory upwards which still contains an __init__.py
-            and whose basename is python-importable.
-            Return None if a pkgpath can not be determined.
-        """
-        pkgpath = None
-        for parent in self.parts(reverse=True):
-            if pkgname is None:
-                if parent.check(file=1):
-                    continue
-                if not isimportable(parent.basename):
-                    break
-                if parent.join('__init__.py').check():
-                    pkgpath = parent
-                    continue
-                return pkgpath
-            else:
-                if parent.basename == pkgname:
-                    return parent
-        return pkgpath
-
-    def _prependsyspath(self, path):
-        s = str(path)
-        if s != sys.path[0]:
-            #print "prepending to sys.path", s
-            sys.path.insert(0, s)
-
     def chmod(self, mode, rec=0):
         """ change permissions to the given mode. If mode is an
             integer it directly encodes the os-specific modes.
@@ -612,33 +592,61 @@ class LocalPath(FSBase):
                 py.error.checked_call(os.chmod, str(x), mode)
         py.error.checked_call(os.chmod, str(self), mode)
 
+    def pypkgpath(self):
+        """ return the Python package path by looking for the last
+        directory upwards which still contains an __init__.py.
+        Return None if a pkgpath can not be determined.
+        """
+        pkgpath = None
+        for parent in self.parts(reverse=True):
+            if parent.isdir():
+                if not parent.join('__init__.py').exists():
+                    break
+                if not isimportable(parent.basename):
+                    break
+                pkgpath = parent
+        return pkgpath
+
+    def _ensuresyspath(self, ensuremode, path):
+        if ensuremode:
+            s = str(path)
+            if ensuremode == "append":
+                if s not in sys.path:
+                    sys.path.append(s)
+            else:
+                if s != sys.path[0]:
+                    sys.path.insert(0, s)
+
     def pyimport(self, modname=None, ensuresyspath=True):
         """ return path as an imported python module.
-            if modname is None, look for the containing package
-            and construct an according module name.
-            The module will be put/looked up in sys.modules.
+
+        If modname is None, look for the containing package
+        and construct an according module name.
+        The module will be put/looked up in sys.modules.
+        if ensuresyspath is True then the root dir for importing
+        the file (taking __init__.py files into account) will
+        be prepended to sys.path if it isn't there already.
+        If ensuresyspath=="append" the root dir will be appended
+        if it isn't already contained in sys.path.
+        if ensuresyspath is False no modification of syspath happens.
         """
         if not self.check():
             raise py.error.ENOENT(self)
-        #print "trying to import", self
+
         pkgpath = None
         if modname is None:
             pkgpath = self.pypkgpath()
             if pkgpath is not None:
-                if ensuresyspath:
-                    self._prependsyspath(pkgpath.dirpath())
-                __import__(pkgpath.basename)
-                pkg = sys.modules[pkgpath.basename]
-                names = self.new(ext='').relto(pkgpath.dirpath())
-                names = names.split(self.sep)
-                if names and names[-1] == "__init__":
+                pkgroot = pkgpath.dirpath()
+                names = self.new(ext="").relto(pkgroot).split(self.sep)
+                if names[-1] == "__init__":
                     names.pop()
                 modname = ".".join(names)
             else:
-                # no package scope, still make it possible
-                if ensuresyspath:
-                    self._prependsyspath(self.dirpath())
+                pkgroot = self.dirpath()
                 modname = self.purebasename
+
+            self._ensuresyspath(ensuresyspath, pkgroot)
             __import__(modname)
             mod = sys.modules[modname]
             if self.basename == "__init__.py":
@@ -652,7 +660,6 @@ class LocalPath(FSBase):
             if modfile.endswith(os.path.sep + "__init__.py"):
                 if self.basename != "__init__.py":
                     modfile = modfile[:-12]
-
             try:
                 issame = self.samefile(modfile)
             except py.error.ENOENT:
@@ -899,8 +906,6 @@ def copychunked(src, dest):
         fsrc.close()
 
 def isimportable(name):
-    if name:
-        if not (name[0].isalpha() or name[0] == '_'):
-            return False
-        name= name.replace("_", '')
+    if name and (name[0].isalpha() or name[0] == '_'):
+        name = name.replace("_", '')
         return not name or name.isalnum()
