@@ -34,6 +34,12 @@ class TestLocalPath(common.CommonFSTests):
         p = tmpdir.join("..//%s/" % tmpdir.basename)
         assert p == tmpdir
 
+    @skiponwin32
+    def test_dirpath_abs_no_abs(self, tmpdir):
+        p = tmpdir.join('foo')
+        assert p.dirpath('/bar') == tmpdir.join('bar')
+        assert tmpdir.dirpath('/bar', abs=True) == py.path.local('/bar')
+
     def test_gethash(self, tmpdir):
         md5 = py.builtin._tryimport('md5', 'hashlib').md5
         lib = py.builtin._tryimport('sha', 'hashlib')
@@ -109,7 +115,8 @@ class TestLocalPath(common.CommonFSTests):
             assert p.check()
 
     @pytest.mark.xfail("sys.version_info < (2,6) and sys.platform == 'win32'")
-    def test_tilde_expansion(self):
+    def test_tilde_expansion(self, monkeypatch, tmpdir):
+        monkeypatch.setenv("HOME", str(tmpdir))
         p = py.path.local("~", expanduser=True)
         assert p == os.path.expanduser("~")
 
@@ -479,6 +486,15 @@ class TestImport:
         m2 = p2.pyimport()
         assert m1 == m2
 
+    def test_ensuresyspath_append(self, tmpdir):
+        root1 = tmpdir.mkdir("root1")
+        file1 = root1.ensure("x123.py")
+        assert str(root1) not in sys.path
+        file1.pyimport(ensuresyspath="append")
+        assert str(root1) == sys.path[-1]
+        assert str(root1) not in sys.path[:-1]
+
+
 def test_pypkgdir(tmpdir):
     pkg = tmpdir.ensure('pkg1', dir=1)
     pkg.ensure("__init__.py")
@@ -581,6 +597,15 @@ class TestWINLocalPath:
             x = py.path.local.sysfind(cmd.relto(root))
             assert x.check(file=1)
 
+    def test_fnmatch_file_abspath_posix_pattern_on_win32(self, tmpdir):
+        # path-matching patterns might contain a posix path separator '/'
+        # Test that we can match that pattern on windows.
+        import posixpath
+        b = tmpdir.join("a", "b")
+        assert b.fnmatch(posixpath.sep.join("ab"))
+        pattern = posixpath.sep.join([str(tmpdir), "*", "b"])
+        assert b.fnmatch(pattern)
+
 class TestPOSIXLocalPath:
     pytestmark = skiponwin32
 
@@ -780,15 +805,19 @@ class TestPOSIXLocalPath:
 
 
 class TestUnicodePy2Py3:
-    def test_join_ensure(self, tmpdir):
+    def test_join_ensure(self, tmpdir, monkeypatch):
+        if sys.version_info >= (3,0) and "LANG" not in os.environ:
+            pytest.skip("cannot run test without locale")
         x = py.path.local(tmpdir.strpath)
-        part = py.builtin._totext("hällo", "utf8")
+        part = "hällo"
         y = x.ensure(part)
         assert x.join(part) == y
 
     def test_listdir(self, tmpdir):
+        if sys.version_info >= (3,0) and "LANG" not in os.environ:
+            pytest.skip("cannot run test without locale")
         x = py.path.local(tmpdir.strpath)
-        part = py.builtin._totext("hällo", "utf8")
+        part = "hällo"
         y = x.ensure(part)
         assert x.listdir(part)[0] == y
 
